"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  KmsProvider: () => KmsProvider
});
module.exports = __toCommonJS(src_exports);

// src/provider.ts
var import_web3_provider_engine = __toESM(require("web3-provider-engine"), 1);
var import_hooked_wallet = __toESM(require("web3-provider-engine/subproviders/hooked-wallet.js"), 1);
var import_rpc = __toESM(require("web3-provider-engine/subproviders/rpc.js"), 1);
var import_ethereumjs_tx = require("ethereumjs-tx");
var import_ethereumjs_common = __toESM(require("ethereumjs-common"), 1);
var import_ethereumjs_util = require("ethereumjs-util");
var import_node_cjs_interop = require("node-cjs-interop");
var import_eth_sig_util = require("@metamask/eth-sig-util");
var import_aws_kms_signer = require("aws-kms-signer");

// src/ethereum.ts
var import_client_js = require("@open-rpc/client-js");
var Ethereum = class {
  constructor(endpoint) {
    this.client = this.getClient(endpoint);
  }
  getClient(url) {
    const transport = new import_client_js.HTTPTransport(url);
    const requestManager = new import_client_js.RequestManager([transport]);
    const client = new import_client_js.Client(requestManager);
    return client;
  }
  async netVersion() {
    const result = await this.client.request({
      method: "net_version",
      params: []
    });
    return result;
  }
};

// src/provider.ts
var Common = (0, import_node_cjs_interop.interopImportCJSDefault)(import_ethereumjs_common.default);
var KmsProvider = class {
  constructor(endpoint, kmsOptions, networkOrNetworkOptions) {
    this.cacheAccounts = [];
    this.engine = new import_web3_provider_engine.default();
    this.signers = kmsOptions.keyIds.map(
      (keyId) => new import_aws_kms_signer.KmsSigner(keyId, {
        credentials: kmsOptions.credential,
        region: kmsOptions.region,
        endpoint: kmsOptions.endpoint
      })
    );
    this.networkOrNetworkOptions = networkOrNetworkOptions;
    this.ethereum = new Ethereum(endpoint);
    this.engine.addProvider(
      new import_hooked_wallet.default({
        getAccounts: (callback) => {
          this.getAccounts().then((accounts) => {
            callback(null, accounts);
          }).catch((err) => {
            callback(err);
          });
        },
        signTransaction: (txData, callback) => {
          this.signTransaction(txData).then((rawTx) => {
            callback(null, rawTx);
          }).catch((err) => callback(err));
        },
        signMessage: (msgParams, callback) => {
          this.signMessage(msgParams).then((signature) => {
            callback(null, signature);
          }).catch((err) => callback(err));
        },
        signTypedMessage: (msgParams, callback) => {
          this.signTypedMessage(msgParams).then((signature) => {
            callback(null, signature);
          }).catch((err) => callback(err));
        }
      })
    );
    this.engine.addProvider(new import_rpc.default({ rpcUrl: endpoint }));
    this.engine.start();
  }
  async getAccounts() {
    if (this.cacheAccounts.length !== 0)
      return this.cacheAccounts;
    const addresses = await Promise.all(
      this.signers.map((signer) => signer.getAddress())
    );
    this.cacheAccounts = addresses.map((address) => `0x${address.toString()}`);
    return this.cacheAccounts;
  }
  async signTransaction(txData) {
    const from = txData.from;
    const signer = this.resolveSigner(from);
    if (!signer) {
      throw new Error(`Account not found: ${from}`);
    }
    const tx = await this.createTransaction(txData);
    const digest = tx.hash(false);
    const signature = await signer.sign(digest);
    const vStr = (signature.v + tx.getChainId() * 2 + 8).toString(16);
    const length = vStr.length + vStr.length % 2;
    const v = Buffer.from(vStr.padStart(length, "0"), "hex");
    tx.r = signature.r;
    tx.s = signature.s;
    tx.v = v;
    const rawTx = `0x${tx.serialize().toString("hex")}`;
    return rawTx;
  }
  async signMessage(msgParams) {
    const from = msgParams.from;
    const signer = this.resolveSigner(from);
    if (!signer) {
      throw new Error(`Account not found: ${from}`);
    }
    const data = (0, import_ethereumjs_util.toBuffer)(msgParams.data);
    const digest = (0, import_ethereumjs_util.hashPersonalMessage)(data);
    const signature = await signer.sign(digest);
    return `0x${signature.toString()}`;
  }
  async signTypedMessage(msgParams) {
    const from = msgParams.from;
    const signer = this.resolveSigner(from);
    if (!signer) {
      throw new Error(`Account not found: ${from}`);
    }
    const typedData = JSON.parse(msgParams.data);
    const digest = Array.isArray(typedData) ? (0, import_ethereumjs_util.toBuffer)((0, import_eth_sig_util.typedSignatureHash)(typedData)) : import_eth_sig_util.TypedDataUtils.eip712Hash(typedData, import_eth_sig_util.SignTypedDataVersion.V4);
    const signature = await signer.sign(digest);
    return `0x${signature.toString()}`;
  }
  send(payload, callback) {
    this.engine.sendAsync(payload, callback);
  }
  sendAsync(payload, callback) {
    this.engine.sendAsync(payload, callback);
  }
  resolveSigner(address) {
    const index = this.cacheAccounts.map((account) => account.toLowerCase()).indexOf(address.toLowerCase());
    if (index !== -1) {
      return this.signers[index];
    }
  }
  async getTransactionOptions() {
    const isNetworkOptions = (x) => {
      return typeof x === "object" && typeof x.chainName === "string" && typeof x.chainId === "number" && typeof x.networkId === "number";
    };
    if (!this.networkOrNetworkOptions) {
      const networkId = Number.parseInt(await this.ethereum.netVersion());
      const common = Common.forCustomChain(
        "mainnet",
        {
          chainId: networkId,
          networkId
        },
        "petersburg"
      );
      return {
        common
      };
    }
    if (isNetworkOptions(this.networkOrNetworkOptions)) {
      const networkOptions = this.networkOrNetworkOptions;
      return {
        common: Common.forCustomChain(
          "mainnet",
          {
            name: networkOptions.chainName,
            chainId: networkOptions.chainId,
            networkId: networkOptions.networkId
          },
          networkOptions.hardfork || "petersburg",
          networkOptions.supportedHardforks
        )
      };
    }
    const newtwork = this.networkOrNetworkOptions;
    return { chain: newtwork };
  }
  async createTransaction(txData) {
    const opts = await this.getTransactionOptions();
    return new import_ethereumjs_tx.Transaction(txData, opts);
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  KmsProvider
});
//# sourceMappingURL=index.cjs.map